from django.db import models
from django.utils import timezone
from django.conf import settings


class AbstractCreateUpdateTime(models.Model):
    created_time = models.DateTimeField(auto_now_add=True)
    updated_time = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class AbstractCreateUpdateUser(AbstractCreateUpdateTime):
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name="%(app_label)s_%(class)s_created_by",
                                   related_query_name="%(app_label)s_%(class)ss",)
    updated_by = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name="%(app_label)s_%(class)s_updated_by",
                                   related_query_name="%(app_label)s_%(class)ss",)

    class Meta:
        abstract = True


class AbstractDeleteTime(AbstractCreateUpdateTime):
    deleted_time = models.DateTimeField(blank=True, null=True)
    is_deleted = models.BooleanField(default=False)

    def __init__(self, *args, **kwargs):
        self.is_deleted_prior = self.is_deleted
        super().__init__(*args, **kwargs)

    def save(self, *args, **kwargs):

        # Toggle Deleted Time Based on Flag
        if self.is_deleted and not self.is_deleted_prior:
            self.deleted_time = timezone.now()
        elif not self.is_deleted and self.deleted_time:
            self.deleted_time = None

        super().save(self, *args, **kwargs)

    class Meta:
        abstract = True


class AbstractDeleteUser(AbstractDeleteTime):
    deleted_by = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                                   related_name="%(app_label)s_%(class)s_deleted_by",
                                   related_query_name="%(app_label)s_%(class)ss",)

    class Meta:
        abstract = True



from .tracking import AbstractDeleteTime
from .uuid import AbstractUUID
from .slug import AbstractUniqueNameSlug
from .other import AbstractOrdering


class AbstractForeignKeySelect(AbstractUniqueNameSlug, AbstractDeleteTime, AbstractUUID, AbstractOrdering):

    class Meta:
        abstract = True



from django.db import models
from autoslug import AutoSlugField


class AbstractUniqueNameSlug(models.Model):
    name = models.CharField(max_length=300, unique=True)
    slug = AutoSlugField(populate_from='name', unique=True)

    def __str__(self):
        return self.name

    class Meta:
        abstract = True



from django.db import models


class AbstractStringPK(models.Model):
    id = models.CharField(max_length=100, primary_key=True)

    class Meta:
        abstract = True


class AbstractOrdering(models.Model):
    default_order = models.IntegerField(default=0)

    class Meta:
        abstract = True
